import 'package:flutter_secure_storage/flutter_secure_storage.dart';
import 'package:hub/shared/helpers/storage/base_storage.dart';

class SecureStorage implements BaseStorage {
  SecureStorage._();

  static final SecureStorage instance = SecureStorage._();

  late final FlutterSecureStorage _secureStorage;
  bool _isInitialized = false;

  @override
  Future<void> init() async {
    if (_isInitialized) return;
    _secureStorage = const FlutterSecureStorage();
    _isInitialized = true;
  }

  @override
  Future<String?> get(String key) async {
    return await _secureStorage.read(key: key);
  }

  @override
  Future<void> set<T>(String key, T value) async {
    await _secureStorage.write(key: key, value: value.toString());
  }

  @override
  Future<void> delete(String key) async {
    await _secureStorage.delete(key: key);
  }

  @override
  Future<void> clearAll() async {
    await _secureStorage.deleteAll();
  }
}
